#include "qcvimageprocessor.h"

QCvImageProcessor::QCvImageProcessor(QObject *parent) : QObject(parent)
{
    camera = Q_NULLPTR;

    algorithm = "gray";
    cvAlgorithm = new QCvAlgorithm();
    workerThread = new QThread();
    connect(cvAlgorithm, &QCvAlgorithm::errorOccured, [this](QString msg)
    {
        emit errorOccured(msg);
    });
    connect(cvAlgorithm, &QCvAlgorithm::imageProcessed, [this](QString fname)
    {
        //fname这个弱类型被emit出来
        emit imageProcessed(fname);
    });
    cvAlgorithm->moveToThread(workerThread);
}

QCvImageProcessor::~QCvImageProcessor()
{
    delete workerThread;
    delete cvAlgorithm;
}

//调用qcamera进行图像采集
void QCvImageProcessor::setCamera(QVariant v)
{
    QObject *o = qvariant_cast<QObject *>(v);
    camera = qvariant_cast<QCamera *>(o->property("mediaObject"));
    camera->setCaptureMode(QCamera::CaptureStillImage);
    imageCapture = new QCameraImageCapture(camera);
    camera->focus()->setFocusMode(QCameraFocus::ContinuousFocus);
    camera->focus()->setFocusPointMode(QCameraFocus::FocusPointAuto);
    //直接在这里设置动作级联
    connect(imageCapture, &QCameraImageCapture::imageSaved, [this](int id, const QString &fileName)
    {
        Q_UNUSED(id);
        processSavedImage(fileName);
    });
}

void QCvImageProcessor::capture()
{
    if(imageCapture->isReadyForCapture())
    {
        //注意这里获得一个可用的图片地址的方法
        imageCapture->capture(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation));
    }
    else
    {
        emit errorOccured("Camera is not ready to capture.");
    }
}

void QCvImageProcessor::processSavedImage(const QString &fname)
{
    if(fname == NULL)
        return;

    emit processStarted();
    qApp->processEvents();
    //使用imageCapture捕获了一个图片，并且将地址放在了inputFileName中
    cvAlgorithm->inputFileName = fname;
    if(algorithm == "none")
    {
        cvAlgorithm->doNone();
    }
    else if(algorithm == "bit")
    {
        cvAlgorithm->doBit();
    }
    else if(algorithm == "canny")
    {
        cvAlgorithm->doCanny();
    }
    else if(algorithm ==  "gray")
    {
        cvAlgorithm->doGray();
    }
    else
    {
        emit errorOccured("Unknown algorithm : " + algorithm);
    }

}

void QCvAlgorithm::doNone()
{
    using namespace cv;
    Mat image = imread(inputFileName.toStdString());

    if( image.empty())
    {
        emit errorOccured("Capture image is empty.");
    }
    else
    {
        imwrite(inputFileName.toStdString(), image);
        emit imageProcessed(inputFileName);
    }
}

void QCvAlgorithm::doGray()
{
    using namespace cv;
    Mat image = imread(inputFileName.toStdString(), CV_LOAD_IMAGE_GRAYSCALE);

    if( image.empty())
    {
        emit errorOccured("Capture image is empty.");
    }
    else
    {
        Mat grayImage;
        cvtColor(image, grayImage, CV_GRAY2BGR);
        imwrite(inputFileName.toStdString(), grayImage);
        emit imageProcessed(inputFileName);
    }
}


void QCvAlgorithm::doBit()
{
    using namespace cv;
    Mat image = imread(inputFileName.toStdString(), CV_LOAD_IMAGE_GRAYSCALE);

    if( image.empty())
    {
        emit errorOccured("Capture image is empty.");
    }
    else
    {
        Mat grayImage;
        threshold(image,image,100,255,THRESH_OTSU);
        //最后保存为了bgr才保存的
        cvtColor(image, grayImage, CV_GRAY2BGR);
        imwrite(inputFileName.toStdString(), grayImage);
        emit imageProcessed(inputFileName);
    }
}

void QCvAlgorithm::doCanny()
{
    using namespace cv;
    Mat image = imread(inputFileName.toStdString());

    if( image.empty())
    {
        emit errorOccured("Capture image is empty.");
    }
    else
    {
        Mat imageGray, dstImage, detectedEdges;
        dstImage.create( image.size(), image.type() );
        cvtColor( image, imageGray, CV_BGR2GRAY );
        blur( imageGray, detectedEdges, Size(3,3) );
        Canny( detectedEdges, detectedEdges, 50, 150, 3 );
        dstImage = Scalar::all(0);
        image.copyTo( dstImage, detectedEdges);
        imwrite(inputFileName.toStdString(), dstImage);
        emit imageProcessed(inputFileName);
    }
}


void QCvImageProcessor::setAlgorithm(QString alg)
{
    algorithm = alg;
    //首次不进入
    if(cvAlgorithm->inputFileName == NULL)
        return;

}
